@@ -21,12 +21,12 @@ from Globals import PersistentMapping
 
 
 from Globals import InitializeClass, DTMLFile
-from Persistence import Persistent
 from AccessControl import ClassSecurityInfo, getSecurityManager
 from Products.CMFCore import CMFCorePermissions
+from Products.CMFCore.ActionProviderBase import ActionProviderBase
 from utils import _dtmldir
 
-class MetadataElementPolicy( Persistent ):
+class MetadataElementPolicy( SimpleItem ):
     """
         Represent a type-specific policy about a particular DCMI element.
     """
@@ -115,7 +115,7 @@ DEFAULT_ELEMENT_SPECS = ( ( 'Title', 0 )
                         , ( 'Rights', 0 )
                         )
 
-class ElementSpec( Persistent ):
+class ElementSpec( SimpleItem ):
     """
         Represent all the tool knows about a single metadata element.
     """
@@ -150,7 +150,7 @@ class ElementSpec( Persistent ):
             object name is 'typ';  return a default, if none found.
         """
         try:
-            return self.policies[ typ ]
+            return self.policies[ typ ].__of__(self)
         except KeyError:
             return self.policies[ None ]
 
@@ -159,7 +159,10 @@ class ElementSpec( Persistent ):
         """
             Return a list of all policies for this element.
         """
-        return self.policies.items()
+        res = []
+        for k, v in self.policies.items():
+            res.append((k, v.__of__(self)))
+        return res
 
     security.declareProtected( CMFCorePermissions.ManagePortal , 'addPolicy' )
     def addPolicy( self, typ ):
@@ -190,11 +193,13 @@ InitializeClass( ElementSpec )
 class MetadataError( Exception ):
     pass
 
-class MetadataTool( UniqueObject, SimpleItem ):
+class MetadataTool( UniqueObject, SimpleItem, ActionProviderBase ):
 
     id              = 'portal_metadata'
     meta_type       = 'Default Metadata Tool'
 
+    _actions = []
+
     security = ClassSecurityInfo()
 
     #
@@ -225,7 +230,8 @@ class MetadataTool( UniqueObject, SimpleItem ):
     #
     #   ZMI methods
     #
-    manage_options = ( ( { 'label'      : 'Overview'
+    manage_options = ( ActionProviderBase.manage_options + 
+                     ( { 'label'      : 'Overview'
                          , 'action'     : 'manage_overview'
                          }
                        , { 'label'      : 'Properties'
@@ -249,6 +255,13 @@ class MetadataTool( UniqueObject, SimpleItem ):
                              , 'propertiesForm' )
     propertiesForm = DTMLFile( 'metadataProperties', _dtmldir )
 
+    security.declarePrivate('listActions')
+    def listActions(self, info=None):
+        """
+        Return actions provided via tool.
+        """
+        return self._actions
+
     security.declareProtected( CMFCorePermissions.ManagePortal
                              , 'editProperties' )
     def editProperties( self
@@ -377,7 +390,10 @@ class MetadataTool( UniqueObject, SimpleItem ):
             Return a list of ElementSpecs representing
             the elements managed by the tool.
         """
-        return tuple( self.element_specs.items() )
+        res = []
+        for k, v in self.element_specs.items():
+            res.append((k, v.__of__(self)))
+        return res
 
     security.declareProtected( CMFCorePermissions.ManagePortal
                              , 'getElementSpec' )
@@ -386,7 +402,7 @@ class MetadataTool( UniqueObject, SimpleItem ):
             Return an ElementSpec representing the tool's knowledge
             of 'element'.
         """
-        return self.element_specs[ element ]
+        return self.element_specs[ element ].__of__( self )
 
     security.declareProtected( CMFCorePermissions.ManagePortal
                              , 'addElementSpec' )
@@ -454,46 +470,47 @@ class MetadataTool( UniqueObject, SimpleItem ):
         return self.publisher
 
     security.declarePublic( 'listAllowedVocabulary' )
-    def listAllowedVocabulary( self, element, content=None ):
+    def listAllowedVocabulary( self, element, content=None, content_type=None ):
         """
-            List allowed keywords for a given meta_type, or all
+            List allowed keywords for a given portal_type, or all
             possible keywords if none supplied.
         """
         spec = self.getElementSpec( element )
-        typ  = content and content.Type() or None
-        return spec.getPolicy( typ ).allowedVocabulary()
+        if content_type is None and content:
+            content_type = content.getPortalTypeName()
+        return spec.getPolicy( content_type ).allowedVocabulary()
 
     security.declarePublic( 'listAllowedSubjects' )
-    def listAllowedSubjects( self, content=None ):
+    def listAllowedSubjects( self, content=None, content_type=None ):
         """
-            List allowed keywords for a given meta_type, or all
+            List allowed keywords for a given portal_type, or all
             possible keywords if none supplied.
         """
-        return self.listAllowedVocabulary( 'Subject', content )
+        return self.listAllowedVocabulary( 'Subject', content, content_type )
 
     security.declarePublic( 'listAllowedFormats' )
-    def listAllowedFormats( self, content=None ):
+    def listAllowedFormats( self, content=None, content_type=None ):
         """
             List the allowed 'Content-type' values for a particular
-            meta_type, or all possible formats if none supplied.
+            portal_type, or all possible formats if none supplied.
         """
-        return self.listAllowedVocabulary( 'Format', content )
+        return self.listAllowedVocabulary( 'Format', content, content_type )
 
     security.declarePublic( 'listAllowedLanguages' )
-    def listAllowedLanguages( self, content=None ):
+    def listAllowedLanguages( self, content=None, content_type=None ):
         """
             List the allowed language values.
         """
-        return self.listAllowedVocabulary( 'Language', content )
+        return self.listAllowedVocabulary( 'Language', content, content_type )
 
     security.declarePublic( 'listAllowedRights' )
-    def listAllowedRights( self, content=None ):
+    def listAllowedRights( self, content=None, content_type=None ):
         """
             List the allowed values for a "Rights"
             selection list;  this gets especially important where
             syndication is involved.
         """
-        return self.listAllowedVocabulary( 'Rights', content )
+        return self.listAllowedVocabulary( 'Rights', content, content_type )
 
     security.declareProtected( CMFCorePermissions.ModifyPortalContent
                              , 'setInitialMetadata' )
@@ -502,7 +519,7 @@ class MetadataTool( UniqueObject, SimpleItem ):
             Set initial values for content metatdata, supplying
             any site-specific defaults.
         """
-        for element, policy in self.listPolicies( content.Type() ):
+        for element, policy in self.listPolicies(content.getPortalTypeName()):
 
             if not getattr( content, element )():
 
@@ -516,7 +533,7 @@ class MetadataTool( UniqueObject, SimpleItem ):
         # TODO:  Call initial_values_hook, if present
 
 
-    security.declareProtected( CMFCorePermissions.ModifyPortalContent
+    security.declareProtected( CMFCorePermissions.View
                              , 'validateMetadata' )
     def validateMetadata( self, content ):
         """
@@ -525,7 +542,7 @@ class MetadataTool( UniqueObject, SimpleItem ):
             by the CMF immediately before saving changes to the
             metadata of an object.
         """
-        for element, policy in self.listPolicies( content.Type() ):
+        for element, policy in self.listPolicies(content.getPortalTypeName()):
 
             value = getattr( content, element )()
             if not value and policy.isRequired():
@@ -535,7 +552,7 @@ class MetadataTool( UniqueObject, SimpleItem ):
             if policy.enforceVocabulary():
                 values = policy.isMultiValued() and value or [ value ]
                 for value in values:
-                    if not value in policy.allowedValues():
+                    if not value in policy.allowedVocabulary():
                         raise MetadataError, \
                         'Value %s is not in allowed vocabulary for' \
                         'metadata element %s.' % ( value, element )
